@@ -24,7 +24,7 @@
 #include <glib/gi18n-lib.h>
 #include "gnome-db-entry-combo.h"
 #include "gnome-db-data-cell-renderer-combo.h"
-#include <libgnomedb/marshal.h>
+#include "marshal.h"
 #include <libgnomedb/gnome-db-combo.h>
 #include <libgnomedb/gnome-db-data-entry.h>
 #include <libgnomedb/utility.h>
