@@ -1,6 +1,6 @@
 /* gnome-db-entry-string.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include "gnome-db-entry-string.h"
+#include "gnome-db-format-entry.h"
 #include <libgda/gda-data-handler.h>
 #include "gdk/gdkkeysyms.h"
 #include <pango/pango.h>
@@ -49,12 +50,14 @@ enum
 {
 	PROP_0,
 	PROP_MULTILINE,
-	PROP_EDITING_CANCELLED
+	PROP_EDITING_CANCELLED,
+	PROP_OPTIONS
 };
 
 /* GtkCellEditable interface */
 static void gnome_db_entry_string_cell_editable_init (GtkCellEditableIface *iface);
 static void gnome_db_entry_string_start_editing (GtkCellEditable *iface, GdkEvent *event);
+static void sync_entry_options (GnomeDbEntryString *mgstr);
 
 /* virtual functions */
 static GtkWidget *create_entry (GnomeDbEntryWrapper *mgwrap);
@@ -65,6 +68,9 @@ static gboolean   expand_in_layout (GnomeDbEntryWrapper *mgwrap);
 static void       set_editable (GnomeDbEntryWrapper *mgwrap, gboolean editable);
 static void       grab_focus (GnomeDbEntryWrapper *mgwrap);
 
+/* options */
+static void set_entry_options (GnomeDbEntryString *mgstr, const gchar *options);
+
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
 
@@ -72,6 +78,7 @@ static GObjectClass  *parent_class = NULL;
 struct _GnomeDbEntryStringPrivate
 {
 	gboolean       multiline;
+	gboolean       is_numeric_type;
 	GtkWidget     *vbox;
 
 	GtkWidget     *entry;
@@ -79,6 +86,13 @@ struct _GnomeDbEntryStringPrivate
 	GtkTextBuffer *buffer;
 	GtkWidget     *sw;
 	GtkWidget     *view;
+
+	gint           maxsize;
+	guchar         thousand_sep;
+	gint           nb_decimals;
+	gchar         *currency;
+
+	gulong         entry_change_sig;
 };
 
 static void
@@ -144,18 +158,28 @@ gnome_db_entry_string_class_init (GnomeDbEntryStringClass * class)
 							       G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (object_class, PROP_EDITING_CANCELLED,
 					 g_param_spec_boolean ("editing_cancelled", NULL, NULL, FALSE, G_PARAM_READABLE));
+	g_object_class_install_property (object_class, PROP_OPTIONS,
+					 g_param_spec_string ("options", NULL, NULL, NULL, G_PARAM_WRITABLE));
 }
 
 static void
-gnome_db_entry_string_init (GnomeDbEntryString * gnome_db_entry_string)
+gnome_db_entry_string_init (GnomeDbEntryString * mgstr)
 {
-	gnome_db_entry_string->priv = g_new0 (GnomeDbEntryStringPrivate, 1);
-	gnome_db_entry_string->priv->multiline = FALSE;
-	gnome_db_entry_string->priv->vbox = NULL;
-	gnome_db_entry_string->priv->entry = NULL;
-	gnome_db_entry_string->priv->buffer = NULL;
-	gnome_db_entry_string->priv->view = NULL;
-	gnome_db_entry_string->priv->sw = NULL;
+	mgstr->priv = g_new0 (GnomeDbEntryStringPrivate, 1);
+	mgstr->priv->multiline = FALSE;
+	mgstr->priv->vbox = NULL;
+	mgstr->priv->entry = NULL;
+	mgstr->priv->buffer = NULL;
+	mgstr->priv->view = NULL;
+	mgstr->priv->sw = NULL;
+
+	mgstr->priv->is_numeric_type = TRUE;
+	mgstr->priv->maxsize = 0; /* unlimited */
+	mgstr->priv->thousand_sep = 0;
+	mgstr->priv->nb_decimals = 0; /* unlimited number of decimals */
+	mgstr->priv->currency = NULL;
+
+	mgstr->priv->entry_change_sig = 0;
 }
 
 /**
@@ -168,7 +192,7 @@ gnome_db_entry_string_init (GnomeDbEntryString * gnome_db_entry_string)
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_entry_string_new (GdaDataHandler *dh, GType type)
+gnome_db_entry_string_new (GdaDataHandler *dh, GType type, const gchar *options)
 {
 	GObject *obj;
 	GnomeDbEntryString *mgstr;
@@ -181,21 +205,39 @@ gnome_db_entry_string_new (GdaDataHandler *dh, GType type)
 	mgstr = GNOME_DB_ENTRY_STRING (obj);
 	gnome_db_data_entry_set_value_type (GNOME_DB_DATA_ENTRY (mgstr), type);
 
+	mgstr->priv->is_numeric_type = FALSE;
+	if ((type == G_TYPE_INT64) || (type == G_TYPE_UINT64) || (type == G_TYPE_DOUBLE) ||
+	    (type == G_TYPE_INT) || (type == GDA_TYPE_NUMERIC) || (type == G_TYPE_FLOAT) ||
+	    (type == GDA_TYPE_SHORT) || (type == GDA_TYPE_USHORT) || (type == G_TYPE_CHAR) ||
+	    (type == G_TYPE_UCHAR) || (type == G_TYPE_LONG) || (type ==  G_TYPE_ULONG) || (type == G_TYPE_UINT))
+		mgstr->priv->is_numeric_type = TRUE;
+	g_object_set (obj, "options", options, NULL);
+
 	return GTK_WIDGET (obj);
 }
 
-
+static gboolean focus_out_cb (GtkWidget *widget, GdkEventFocus *event, GnomeDbEntryString *mgstr);
 static void
 gnome_db_entry_string_dispose (GObject   * object)
 {
-	GnomeDbEntryString *gnome_db_entry_string;
+	GnomeDbEntryString *mgstr;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_DB_IS_ENTRY_STRING (object));
 
-	gnome_db_entry_string = GNOME_DB_ENTRY_STRING (object);
-	if (gnome_db_entry_string->priv) {
-
+	mgstr = GNOME_DB_ENTRY_STRING (object);
+	if (mgstr->priv) {
+		if (mgstr->priv->entry) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (mgstr->priv->entry),
+							      G_CALLBACK (focus_out_cb), mgstr);
+			mgstr->priv->entry = NULL;
+		}
+		
+		if (mgstr->priv->view) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (mgstr->priv->view),
+							      G_CALLBACK (focus_out_cb), mgstr);
+			mgstr->priv->view = NULL;
+		}
 	}
 
 	/* parent class */
@@ -205,16 +247,16 @@ gnome_db_entry_string_dispose (GObject   * object)
 static void
 gnome_db_entry_string_finalize (GObject   * object)
 {
-	GnomeDbEntryString *gnome_db_entry_string;
+	GnomeDbEntryString *mgstr;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_DB_IS_ENTRY_STRING (object));
 
-	gnome_db_entry_string = GNOME_DB_ENTRY_STRING (object);
-	if (gnome_db_entry_string->priv) {
-
-		g_free (gnome_db_entry_string->priv);
-		gnome_db_entry_string->priv = NULL;
+	mgstr = GNOME_DB_ENTRY_STRING (object);
+	if (mgstr->priv) {
+		g_free (mgstr->priv->currency);
+		g_free (mgstr->priv);
+		mgstr->priv = NULL;
 	}
 
 	/* parent class */
@@ -223,9 +265,9 @@ gnome_db_entry_string_finalize (GObject   * object)
 
 static void
 gnome_db_entry_string_set_property (GObject *object,
-			      guint param_id,
-			      const GValue *value,
-			      GParamSpec *pspec)
+				    guint param_id,
+				    const GValue *value,
+				    GParamSpec *pspec)
 {
 	GnomeDbEntryString *mgstr;
 
@@ -233,6 +275,9 @@ gnome_db_entry_string_set_property (GObject *object,
 	if (mgstr->priv) {
 		switch (param_id) {
 		case PROP_MULTILINE:
+			if (mgstr->priv->is_numeric_type)
+				return;
+
 			if (g_value_get_boolean (value) != mgstr->priv->multiline) {
 				mgstr->priv->multiline = g_value_get_boolean (value);
 				if (mgstr->priv->multiline) {
@@ -245,6 +290,9 @@ gnome_db_entry_string_set_property (GObject *object,
 				}
 			}
 			break;
+		case PROP_OPTIONS:
+			set_entry_options (mgstr, g_value_get_string (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -292,7 +340,8 @@ create_entry (GnomeDbEntryWrapper *mgwrap)
 	mgstr->priv->vbox = vbox;
 
 	/* one line entry */
-	mgstr->priv->entry = gtk_entry_new ();
+	mgstr->priv->entry = gnome_db_format_entry_new ();
+	sync_entry_options (mgstr);
 	gtk_box_pack_start (GTK_BOX (vbox), mgstr->priv->entry, FALSE, TRUE, 0);
 	g_signal_connect_after (G_OBJECT (mgstr->priv->entry), "show", 
 				G_CALLBACK (widget_shown_cb), mgstr);
@@ -356,7 +405,7 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 	/* fill the single line widget */
 	if (value) {
 		if (gda_value_is_null ((GValue *) value))
-			gtk_entry_set_text (GTK_ENTRY (mgstr->priv->entry), "");
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), NULL);
 		else {
 			gchar *str;
 
@@ -371,15 +420,15 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 					memmove (str + len, str, MAX_ACCEPTED_STRING_LENGTH - len);
 					memcpy (str, msg, strlen (msg));
 				}
-				gtk_entry_set_text (GTK_ENTRY (mgstr->priv->entry), str);
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), str);
 				g_free (str);
 			}
 			else
-				gtk_entry_set_text (GTK_ENTRY (mgstr->priv->entry), "");
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), NULL);
 		}
 	}
 	else 
-		gtk_entry_set_text (GTK_ENTRY (mgstr->priv->entry), "");
+		gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry), NULL);
 
 	/* fill the multiline widget */
 	if (value) {
@@ -413,9 +462,10 @@ real_get_value (GnomeDbEntryWrapper *mgwrap)
 
 	dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
 	if (! mgstr->priv->multiline) {
-		const gchar *cstr;
-		cstr = gtk_entry_get_text (GTK_ENTRY (mgstr->priv->entry));
+		gchar *cstr;
+		cstr = gnome_db_format_entry_get_text (GNOME_DB_FORMAT_ENTRY (mgstr->priv->entry));
 		value = gda_data_handler_get_value_from_str (dh, cstr, gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgwrap)));
+		g_free (cstr);
 	}
 	else {
 		GtkTextIter start, end;
@@ -459,8 +509,8 @@ connect_signals(GnomeDbEntryWrapper *mgwrap, GCallback modify_cb, GCallback acti
 	g_object_set_data (G_OBJECT (mgstr->priv->entry), "_activate_cb", activate_cb);
 	g_object_set_data (G_OBJECT (mgstr->priv->view), "_activate_cb", activate_cb);
 
-	g_signal_connect (G_OBJECT (mgstr->priv->entry), "changed",
-			  modify_cb, mgwrap);
+	mgstr->priv->entry_change_sig = g_signal_connect (G_OBJECT (mgstr->priv->entry), "changed",
+							  modify_cb, mgwrap);
 	g_signal_connect (G_OBJECT (mgstr->priv->entry), "activate",
 			  activate_cb, mgwrap);
 	g_signal_connect (G_OBJECT (mgstr->priv->entry), "focus-out-event",
@@ -555,3 +605,98 @@ gnome_db_entry_string_start_editing (GtkCellEditable *iface, GdkEvent *event)
 	gtk_widget_grab_focus (mgstr->priv->entry);
 	gtk_widget_queue_draw (GTK_WIDGET (mgstr));
 }
+
+/*
+ * Options handling
+ */
+
+static guchar
+get_default_thousands_sep ()
+{
+	static guchar value = 255;
+
+	if (value == 255) {
+		gchar text[20];
+		sprintf (text, "%f", 1234.);
+		if (text[1] == '2')
+			value = 0;
+		else
+			value = text[1];	
+	}
+	return value;
+}
+
+static void
+set_entry_options (GnomeDbEntryString *mgstr, const gchar *options)
+{
+	g_assert (mgstr->priv);
+
+	if (options && *options) {
+                GdaQuarkList *params;
+                const gchar *str;
+		
+                params = gda_quark_list_new_from_string (options);
+		if (!mgstr->priv->is_numeric_type) {
+			/* STRING specific options */
+			str = gda_quark_list_find (params, "MAX_SIZE");
+			if (str) 
+				mgstr->priv->maxsize = atoi (str);
+
+			str = gda_quark_list_find (params, "MULTILINE");
+			if (str) {
+				if ((*str == 't') || (*str == 'T'))
+					mgstr->priv->multiline = TRUE;
+				else
+					mgstr->priv->multiline = FALSE;
+				
+				if (mgstr->priv->entry) {
+					if (mgstr->priv->multiline) {
+						gtk_widget_hide (mgstr->priv->entry);
+						gtk_widget_show (mgstr->priv->sw);
+					}
+					else {
+						gtk_widget_show (mgstr->priv->entry);
+						gtk_widget_hide (mgstr->priv->sw);
+					}
+				}
+			}
+                }
+		else {
+			/* NUMERIC specific options */
+			str = gda_quark_list_find (params, "THOUSAND_SEP");
+			if (str) {
+				if ((*str == 't') || (*str == 'T'))
+					mgstr->priv->thousand_sep = get_default_thousands_sep ();
+				else
+					mgstr->priv->thousand_sep = 0;
+			}
+			str = gda_quark_list_find (params, "NB_DECIMALS");
+			if (str) 
+				mgstr->priv->nb_decimals = atoi (str);
+			str = gda_quark_list_find (params, "CURRENCY");
+			if (str) {
+				g_free (mgstr->priv->currency);
+				mgstr->priv->currency = g_strdup_printf ("%s ", str);
+			}
+		}
+                gda_quark_list_free (params);
+		sync_entry_options (mgstr);
+        }
+}
+
+/* sets the correct options for mgstr->priv->entry if it exists */
+static void
+sync_entry_options (GnomeDbEntryString *mgstr)
+{
+	if (!mgstr->priv->entry)
+		return;
+
+	g_object_set (G_OBJECT (mgstr->priv->entry), 
+		      "edited_type", gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgstr)),
+		      "n_decimals", mgstr->priv->nb_decimals,
+		      "thousands_sep", mgstr->priv->thousand_sep,
+		      "prefix", mgstr->priv->currency,
+		      "max_length", mgstr->priv->maxsize,
+		      NULL);
+	g_signal_emit_by_name (mgstr->priv->entry, "changed");
+}
