@@ -39,11 +39,26 @@ struct _GnomeDbConnectionPropertiesPrivate {
 	GtkWidget *feature_list;
 };
 
+enum {
+	PROP_0,
+
+	PROP_CNC
+};
+
 static void gnome_db_connection_properties_class_init (GnomeDbConnectionPropertiesClass *klass);
 static void gnome_db_connection_properties_init       (GnomeDbConnectionProperties *props,
 						       GnomeDbConnectionPropertiesClass *klass);
 static void gnome_db_connection_properties_finalize   (GObject *object);
 
+static void gnome_db_connection_properties_set_property (GObject *object,
+                                                         guint param_id,
+                                                         const GValue *value,
+                                                         GParamSpec *pspec);
+static void gnome_db_connection_properties_get_property (GObject* object,
+                                                         guint param_id,
+                                                         GValue *value,
+                                                         GParamSpec *pspec);
+
 static GObjectClass *parent_class = NULL;
 
 /*
@@ -77,6 +92,13 @@ gnome_db_connection_properties_class_init (GnomeDbConnectionPropertiesClass *kla
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gnome_db_connection_properties_finalize;
+	object_class->set_property = gnome_db_connection_properties_set_property;
+	object_class->get_property = gnome_db_connection_properties_get_property;
+
+	g_object_class_install_property (object_class, PROP_CNC,
+	                                 g_param_spec_object("connection", NULL, NULL,
+					                     GDA_TYPE_CONNECTION,
+							     G_PARAM_READWRITE));
 }
 
 static void
@@ -177,6 +199,42 @@ gnome_db_connection_properties_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+static void gnome_db_connection_properties_set_property (GObject *object,
+                                                         guint param_id,
+                                                         const GValue *value,
+                                                         GParamSpec *pspec)
+{
+	GnomeDbConnectionProperties *properties;
+	GdaConnection *cnc;
+
+	properties = GNOME_DB_CONNECTION_PROPERTIES (object);
+
+	switch (param_id)
+	{
+	case PROP_CNC:
+		cnc = GDA_CONNECTION (g_value_get_object (value));
+		gnome_db_connection_properties_set_connection (properties, cnc);
+		break;
+	}
+}
+
+static void gnome_db_connection_properties_get_property (GObject* object,
+                                                         guint param_id,
+                                                         GValue *value,
+                                                         GParamSpec *pspec)
+{
+	GnomeDbConnectionProperties *properties;
+
+	properties = GNOME_DB_CONNECTION_PROPERTIES (object);
+
+	switch (param_id)
+	{
+	case PROP_CNC:
+		g_value_set_object(value, G_OBJECT(properties->priv->cnc));
+		break;
+	}
+}
+
 GType
 gnome_db_connection_properties_get_type (void)
 {
@@ -214,12 +272,8 @@ gnome_db_connection_properties_new (GdaConnection *cnc)
 {
 	GnomeDbConnectionProperties *props;
 
-	props = g_object_new (GNOME_DB_TYPE_CONNECTION_PROPERTIES, NULL);
-	if (GDA_IS_CONNECTION (cnc)) {
-		g_object_ref (G_OBJECT (cnc));
-		props->priv->cnc = cnc;
-		refresh_widget (props);
-	}
+	props = g_object_new (GNOME_DB_TYPE_CONNECTION_PROPERTIES,
+	                      "connection", cnc, NULL);
 
 	return GTK_WIDGET (props);
 }
@@ -257,4 +311,6 @@ gnome_db_connection_properties_set_connection (GnomeDbConnectionProperties *prop
 		g_object_ref (G_OBJECT (props->priv->cnc));
 		refresh_widget (props);
 	}
+
+	g_object_notify (G_OBJECT (props), "connection");
 }
