@@ -1,6 +1,6 @@
 /* gnome-db-data-entry.c
  *
- * Copyright (C) 2003 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,11 +44,13 @@
 enum
 {
 	CONTENTS_MODIFIED,
+	CONTENTS_ACTIVATED,
 	STATUS_CHANGED,
+	CONTENTS_VALID,
 	LAST_SIGNAL
 };
 
-static gint gnome_db_data_entry_signals[LAST_SIGNAL] = { 0 };
+static gint gnome_db_data_entry_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 static void gnome_db_data_entry_iface_init (gpointer g_class);
 
 GType
@@ -74,6 +76,25 @@ gnome_db_data_entry_get_type (void)
 	return type;
 }
 
+static gboolean
+contents_valid_accumulator (GSignalInvocationHint *ihint,
+		   GValue *return_accu,
+		   const GValue *handler_return,
+		   gpointer data)
+{
+        gboolean thisvalue;
+
+        thisvalue = g_value_get_boolean (handler_return);
+        g_value_set_boolean (return_accu, thisvalue); 
+
+        return thisvalue; /* stop signal if 'thisvalue' is FALSE */
+}
+
+static gboolean
+m_class_contents_valid (GnomeDbDataEntry *de, GError **error)
+{
+	return TRUE;
+}
 
 static void
 gnome_db_data_entry_iface_init (gpointer g_class)
@@ -89,6 +110,14 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 				      NULL, NULL,
 				      marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
+		gnome_db_data_entry_signals[CONTENTS_ACTIVATED] =
+			g_signal_new ("contents_activated",
+				      GNOME_DB_TYPE_DATA_ENTRY,
+				      G_SIGNAL_RUN_FIRST,
+				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, contents_activated),
+				      NULL, NULL,
+				      marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
 		gnome_db_data_entry_signals[STATUS_CHANGED] =
 			g_signal_new ("status_changed",
 				      GNOME_DB_TYPE_DATA_ENTRY,
@@ -97,7 +126,16 @@ gnome_db_data_entry_iface_init (gpointer g_class)
 				      NULL, NULL,
 				      marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
+		gnome_db_data_entry_signals[CONTENTS_VALID] =
+			g_signal_new ("contents_valid",
+				      GNOME_DB_TYPE_DATA_ENTRY,
+				      G_SIGNAL_RUN_FIRST,
+				      G_STRUCT_OFFSET (GnomeDbDataEntryIface, contents_valid),
+				      contents_valid_accumulator, NULL,
+				      marshal_BOOLEAN__POINTER,
+				      G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
+		((GnomeDbDataEntryIface*) g_class)->contents_valid = m_class_contents_valid;
 		initialized = TRUE;
 	}
 }
@@ -111,10 +149,10 @@ gnome_db_data_entry_iface_init (gpointer g_class)
  * the widget can handle.
  */
 void
-gnome_db_data_entry_set_value_type (GnomeDbDataEntry *de, GdaValueType type)
+gnome_db_data_entry_set_value_type (GnomeDbDataEntry *de, GType type)
 {
-	g_return_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de));
-	g_return_if_fail (type != GDA_VALUE_TYPE_UNKNOWN);
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
+	g_return_if_fail (type != G_TYPE_INVALID);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_type)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_type) (de, type);
@@ -127,32 +165,31 @@ gnome_db_data_entry_set_value_type (GnomeDbDataEntry *de, GdaValueType type)
  *
  * Fetch the type of data the GnomeDbDataEntry handles
  *
- * Returns: the GdaValueType type
+ * Returns: the GType type
  */
-GdaValueType
+GType
 gnome_db_data_entry_get_value_type (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de), GDA_VALUE_TYPE_UNKNOWN);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), G_TYPE_INVALID);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_type)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_type) (de);
-	return GDA_VALUE_TYPE_UNKNOWN;
+	return G_TYPE_INVALID;
 }
 
-
 /**
  * gnome_db_data_entry_set_value
  * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
  * @value: 
  *
  * Push a value into the GnomeDbDataEntry. The value parameter must either be:
- * - NULL or of type GDA_VALUE_TYPE_NULL, or
+ * - NULL or of type GDA_TYPE_NULL, or
  * - of type specified using gnome_db_data_entry_set_value_type.
  */
 void
-gnome_db_data_entry_set_value (GnomeDbDataEntry *de, const GdaValue * value)
+gnome_db_data_entry_set_value (GnomeDbDataEntry *de, const GValue * value)
 {
-	g_return_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value) (de, value);
@@ -163,16 +200,16 @@ gnome_db_data_entry_set_value (GnomeDbDataEntry *de, const GdaValue * value)
  * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
  *
  * Fetch the value held in the GnomeDbDataEntry widget. If the value is set to NULL,
- * the returned value is of type GDA_VALUE_TYPE_NULL. If the value is set to default,
- * then the returned value is of type GDA_VALUE_TYPE_NULL or is the default value if it
+ * the returned value is of type GDA_TYPE_NULL. If the value is set to default,
+ * then the returned value is of type GDA_TYPE_NULL or is the default value if it
  * has been provided to the widget (and is of the same type as the one provided by @de).
  *
- * Returns: a new GdaValue
+ * Returns: a new GValue
  */
-GdaValue *
+GValue *
 gnome_db_data_entry_get_value (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), NULL);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value) (de);
@@ -180,6 +217,29 @@ gnome_db_data_entry_get_value (GnomeDbDataEntry *de)
 	return NULL;
 }
 
+/**
+ * gnome_db_data_entry_is_contents_valid
+ * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
+ * @error: a place to store an error, or %NULL
+ *
+ * Tests the validity of @de's contents. The validity is a determined from:
+ * <itemizedlist>
+ * <listitem><para>the @de widget itself if it is capable of doing it (depending on the implementation)</para></listitem>
+ * <listitem><para>the results of the "contents_valid" signal which can be connected from </para></listitem>
+ * </itemizedlist>
+ *
+ * Returns: TRUE if @de's contents is valid
+ */
+gboolean
+gnome_db_data_entry_is_contents_valid (GnomeDbDataEntry *de, GError **error)
+{
+	gboolean is_valid;
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), FALSE);
+
+	g_signal_emit (de, gnome_db_data_entry_signals [CONTENTS_VALID], 0, error, &is_valid);
+	return is_valid;
+}
+
 
 /**
  * gnome_db_data_entry_set_value_orig
@@ -190,9 +250,9 @@ gnome_db_data_entry_get_value (GnomeDbDataEntry *de)
  * also sets this value as the original value.
  */
 void
-gnome_db_data_entry_set_value_orig (GnomeDbDataEntry *de, const GdaValue *value)
+gnome_db_data_entry_set_value_orig (GnomeDbDataEntry *de, const GValue *value)
 {
-	g_return_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_orig)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_orig) (de, value);
@@ -207,8 +267,8 @@ gnome_db_data_entry_set_value_orig (GnomeDbDataEntry *de, const GdaValue *value)
 void
 gnome_db_data_entry_set_current_as_orig (GnomeDbDataEntry *de)
 {
-	GdaValue *value;
-	g_return_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de));
+	GValue *value;
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 	
 	value = gnome_db_data_entry_get_value (de);
 	gnome_db_data_entry_set_value_orig (de, value);
@@ -223,12 +283,12 @@ gnome_db_data_entry_set_current_as_orig (GnomeDbDataEntry *de)
  *
  * Fetch the original value held in the GnomeDbDataEntry widget
  *
- * Returns: the GdaValue
+ * Returns: the GValue
  */
-const GdaValue *
+const GValue *
 gnome_db_data_entry_get_value_orig (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), NULL);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_orig)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_value_orig) (de);
@@ -243,15 +303,15 @@ gnome_db_data_entry_get_value_orig (GnomeDbDataEntry *de)
  * @value: 
  *
  * Sets the default value for the GnomeDbDataEntry which gets displayed when the
- * user forces the default value. If it is not set then it is set to type GDA_VALUE_TYPE_NULL.
+ * user forces the default value. If it is not set then it is set to type GDA_TYPE_NULL.
  * The value parameter must either be:
- * - NULL or of type GDA_VALUE_TYPE_NULL, or
+ * - NULL or of type GDA_TYPE_NULL, or
  * - of type specified using gnome_db_data_entry_set_value_type.
  */
 void
-gnome_db_data_entry_set_value_default (GnomeDbDataEntry *de, const GdaValue *value)
+gnome_db_data_entry_set_value_default (GnomeDbDataEntry *de, const GValue *value)
 {
-	g_return_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 	g_return_if_fail (value);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_value_default)
@@ -268,9 +328,9 @@ gnome_db_data_entry_set_value_default (GnomeDbDataEntry *de, const GdaValue *val
  * mask are set, the other ones are ignored.
  */
 void
-gnome_db_data_entry_set_attributes (GnomeDbDataEntry *de, guint attrs, guint mask)
+gnome_db_data_entry_set_attributes (GnomeDbDataEntry *de, GdaValueAttribute attrs, GdaValueAttribute mask)
 {
-	g_return_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de));
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_attributes)
 		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_attributes) (de, attrs, mask);
@@ -284,10 +344,10 @@ gnome_db_data_entry_set_attributes (GnomeDbDataEntry *de, guint attrs, guint mas
  *
  * Returns: the OR'ed bits corresponding to the attributes.
  */
-guint
+GdaValueAttribute
 gnome_db_data_entry_get_attributes (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de), 0);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), 0);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_attributes)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_attributes) (de);
@@ -300,14 +360,14 @@ gnome_db_data_entry_get_attributes (GnomeDbDataEntry *de)
  * gnome_db_data_entry_get_handler
  * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
  * 
- * Fetch the GnomeDbDataHandler the GnomeDbDataEntry is using
+ * Fetch the GdaDataHandler the GnomeDbDataEntry is using
  *
- * Returns: the GnomeDbDataHandler object
+ * Returns: the GdaDataHandler object
  */
-GnomeDbDataHandler  *
+GdaDataHandler  *
 gnome_db_data_entry_get_handler (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), NULL);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_handler)
 		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->get_handler) (de);
@@ -326,11 +386,43 @@ gnome_db_data_entry_get_handler (GnomeDbDataEntry *de)
 gboolean
 gnome_db_data_entry_expand_in_layout (GnomeDbDataEntry *de)
 {
-	g_return_val_if_fail (de && IS_GNOME_DB_DATA_ENTRY (de), FALSE);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_ENTRY (de), FALSE);
 
 	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->expand_in_layout)
-		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->expand_in_layout) (de);
-
-	return FALSE;
+		return (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->expand_in_layout) (de);
+	else
+		return FALSE;
 }
 
+/**
+ * gnome_db_data_entry_set_editable
+ * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
+ * @editable:
+ *
+ * Set if @de can be modified or not by the user
+ */
+void
+gnome_db_data_entry_set_editable (GnomeDbDataEntry *de, gboolean editable)
+{
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
+
+	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_editable)
+		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->set_editable) (de, editable);
+	else
+		gtk_widget_set_sensitive (GTK_WIDGET (de), editable);
+}
+
+/**
+ * gnome_db_data_entry_grab_focus
+ * @de: a #GtkWidget object which implements the #GnomeDbDataEntry interface
+ *
+ * Makes @de grab the focus for the window it's in
+ */
+void
+gnome_db_data_entry_grab_focus (GnomeDbDataEntry *de)
+{
+	g_return_if_fail (GNOME_DB_IS_DATA_ENTRY (de));
+
+	if (GNOME_DB_DATA_ENTRY_GET_IFACE (de)->grab_focus)
+		(GNOME_DB_DATA_ENTRY_GET_IFACE (de)->grab_focus) (de);
+}
