@@ -24,6 +24,7 @@
 #include "gnome-db-data-widget.h"
 #include "gnome-db-raw-grid.h"
 #include "gnome-db-data-widget-info.h"
+#include "gnome-db-enum-types.h"
 
 static void gnome_db_data_widget_info_class_init (GnomeDbDataWidgetInfoClass * class);
 static void gnome_db_data_widget_info_init (GnomeDbDataWidgetInfo *wid);
@@ -54,7 +55,7 @@ struct _GnomeDbDataWidgetInfoPriv
 	GnomeDbDataWidget *data_widget;
 	GdaDataProxy      *proxy;
 	GdaDataModelIter  *iter;
-	guint              flags;
+	GnomeDbDataWidgetInfoFlag flags; /* ORed values. */
 
 	GtkUIManager      *uimanager;
 
@@ -115,7 +116,7 @@ gnome_db_data_widget_info_class_init (GnomeDbDataWidgetInfoClass * class)
                                          g_param_spec_object ("data_widget", NULL, NULL, GNOME_DB_TYPE_DATA_WIDGET,
                                                                G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (object_class, PROP_FLAGS,
-                                         g_param_spec_uint ("flags", NULL, NULL, 0, G_MAXUINT, 
+                                         g_param_spec_flags ("flags", NULL, NULL, GNOME_DB_TYPE_DB_DATA_WIDGET_INFO_FLAG, 
 							    GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW,
 							    G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
@@ -131,14 +132,14 @@ gnome_db_data_widget_info_init (GnomeDbDataWidgetInfo * wid)
 /**
  * gnome_db_data_widget_info_new
  * @data_widget: a widget implementing the #GnomeDbDataWidget interface
- * @flags: what to display in the new widget
+ * @flags: OR'ed values, specifying what to display in the new widget
  *
  * Creates a new #GnomeDbDataWidgetInfo widget suitable to display information about @data_widget
  *
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_data_widget_info_new (GnomeDbDataWidget *data_widget, guint flags)
+gnome_db_data_widget_info_new (GnomeDbDataWidget *data_widget, GnomeDbDataWidgetInfoFlag flags)
 {
 	GtkWidget *info;
 
@@ -289,7 +290,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 			}
                         break;
                 case PROP_FLAGS:
-			info->priv->flags = g_value_get_uint (value);
+			info->priv->flags = g_value_get_flags (value);
 			if (info->priv->buttons_bar) {
 				gtk_widget_destroy (info->priv->buttons_bar);
 				info->priv->buttons_bar = NULL;
@@ -328,7 +329,7 @@ gnome_db_data_widget_info_get_property (GObject *object,
 			g_value_set_pointer (value, info->priv->data_widget);
 			break;
 		case PROP_FLAGS:
-			g_value_set_uint (value, info->priv->flags);
+			g_value_set_flags (value, info->priv->flags);
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -407,12 +408,12 @@ static void
 modif_buttons_make (GnomeDbDataWidgetInfo *info)
 {
 	GtkWidget *wid;
-	guint flags = info->priv->flags;
+	GnomeDbDataWidgetInfoFlag flags = info->priv->flags;
 
 	if (! info->priv->data_widget)
 		return;
 
-	if (flags & (GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS |
+	if (flags & (GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS |
 		     GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS |
 		     GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS)) {
 		GtkActionGroup *actions;
@@ -421,7 +422,7 @@ modif_buttons_make (GnomeDbDataWidgetInfo *info)
 		actions = gnome_db_data_widget_get_actions_group (info->priv->data_widget);
 		ui = gtk_ui_manager_new ();
 		gtk_ui_manager_insert_action_group (ui, actions, 0);
-		if (flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS)
+		if (flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS)
 			gtk_ui_manager_add_ui_from_string (ui, ui_row_modif, -1, NULL);
 		if (flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS)
 			gtk_ui_manager_add_ui_from_string (ui, ui_row_move, -1, NULL);
@@ -496,7 +497,7 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 
 	GtkAction *action;
 	gint sample_first_row = 0, sample_last_row = 0, sample_size = 0;
-	guint flags = 0;
+	GnomeDbDataWidgetInfoFlag flags = 0;
 	
 	model_iter = gnome_db_data_widget_get_current_data (info->priv->data_widget);
 	if (info->priv->proxy) {
@@ -582,7 +583,7 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 			}
 			else 
 				if (GNOME_DB_IS_RAW_GRID (info->priv->data_widget)) {
-					/* bad for encapsulation, but very usefull... */
+					/* bad for encapsulation, but very useful... */
 					GList *sel, *list;
 					
 					sel = gnome_db_raw_grid_get_selection ((GnomeDbRawGrid*) info->priv->data_widget);
@@ -610,7 +611,7 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 				}
 		}
 
-		if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS) {
+		if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS) {
 			GnomeDbDataWidgetWriteMode mode;
 			mode = gnome_db_data_widget_get_write_mode (info->priv->data_widget);
 
