@@ -24,6 +24,7 @@
 #if !defined(__gnome_db_login_dialog_h__)
 #  define __gnome_db_login_dialog_h__
 
+#include <libgnomedb/gnome-db-login.h>
 #include <gtk/gtkdialog.h>
 
 G_BEGIN_DECLS
@@ -54,7 +55,7 @@ const gchar *gnome_db_login_dialog_get_dsn          (GnomeDbLoginDialog *dialog)
 const gchar *gnome_db_login_dialog_get_username     (GnomeDbLoginDialog *dialog);
 const gchar *gnome_db_login_dialog_get_password     (GnomeDbLoginDialog *dialog);
 
-GtkWidget   *gnome_db_login_dialog_get_login_widget (GnomeDbLoginDialog *dialog);
+GnomeDbLogin   *gnome_db_login_dialog_get_login_widget (GnomeDbLoginDialog *dialog);
 
 G_END_DECLS
 
