@@ -25,8 +25,12 @@
 #include <libgda/gda-value.h>
 #include "gnome-db-decl.h"
 
+G_BEGIN_DECLS
+
 void              gnome_db_util_init_plugins      ();
 GnomeDbDataEntry *gnome_db_util_new_data_entry    (GdaDict *dict, GType type, const gchar *plugin_name);
 GtkCellRenderer  *gnome_db_util_new_cell_renderer (GdaDict *dict, GType type, const gchar *plugin_name);
 
+G_END_DECLS
+
 #endif
